<?php
/**
 * Perfect Checkout - Save tokens controller.
 *
 * Handles the request to save a payment token to the customers account.
 *
 * @author  Sébastien Dumont
 * @package Perfect Checkout\API
 * @since   1.3.0
 * @version 1.4.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * REST API Save tokens controller class.
 *
 * @package Perfect Checkout\API
 */
class Perfect_Checkout_API_Save_Tokens_Controller {

	/**
	 * Endpoint namespace.
	 *
	 * @var string
	 */
	protected $namespace = 'pc/connect';

	/**
	 * Route base.
	 *
	 * @var string
	 */
	protected $rest_base = 'save-tokens';

	/**
	 * Register routes.
	 *
	 * @access public
	 */
	public function register_routes() {
		register_rest_route(
			$this->namespace,
			'/' . $this->rest_base,
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'save_token_callback' ),
				'permission_callback' => array( $this, 'update_items_permissions_check' ),
				'args'                => array(
					'source_id' => array(
						'required'          => true,
						'description'       => __( 'Source ID of the token.', 'perfect-checkout' ),
						'type'              => 'string',
						'sanitize_callback' => 'sanitize_text_field',
						'validate_callback' => 'rest_validate_request_arg',
					),
					'user_id' => array(
						'required'          => true,
						'description'       => __( 'Customers user ID.', 'perfect-checkout' ),
						'type'              => 'string',
						'sanitize_callback' => 'sanitize_text_field',
						'validate_callback' => 'rest_validate_request_arg',
					),
					'cc_type' => array(
						'required'          => true,
						'description'       => __( 'Card type.', 'perfect-checkout' ),
						'type'              => 'string',
						'sanitize_callback' => 'sanitize_text_field',
						'validate_callback' => 'rest_validate_request_arg',
					),
					'payment_type' => array(
						'required'          => false,
						'description'       => __( 'Payment type.', 'perfect-checkout' ),
						'type'              => 'string',
						'sanitize_callback' => 'sanitize_text_field',
						'validate_callback' => 'rest_validate_request_arg',
					),
					'gateway' => array(
						'required'          => false,
						'description'       => __( 'Payment gateway.', 'perfect-checkout' ),
						'type'              => 'string',
						'sanitize_callback' => 'sanitize_text_field',
						'validate_callback' => 'rest_validate_request_arg',
					),
					'last_4' => array(
						'required'          => true,
						'description'       => __( 'Last 4 digits of card.', 'perfect-checkout' ),
						'type'              => 'string',
						'sanitize_callback' => 'sanitize_text_field',
						'validate_callback' => 'rest_validate_request_arg',
					),
					'exp_month' => array(
						'required'          => false,
						'description'       => __( 'Expiry month.', 'perfect-checkout' ),
						'type'              => 'string',
						'sanitize_callback' => 'sanitize_text_field',
						'validate_callback' => 'rest_validate_request_arg',
					),
					'exp_year' => array(
						'required'          => false,
						'description'       => __( 'Expiry year.', 'perfect-checkout' ),
						'type'              => 'string',
						'sanitize_callback' => 'sanitize_text_field',
						'validate_callback' => 'rest_validate_request_arg',
					),
				),
			)
		);
	} // register_routes()

	/**
	 * Need write/create permission.
	 *
	 * @access public
	 */
	public function update_items_permissions_check() {
		if ( ! wc_rest_check_user_permissions( 'create' ) ) {
			return new WP_Error( 'perfectcheckout_rest_cannot_create', __( 'Sorry, you are not allowed to create resources.', 'perfect-checkout' ), array( 'status' => rest_authorization_required_code() ) );
		}

		return true;
	} // END update_items_permissions_check()

	/**
	 * Saves payment token to customers account.
	 *
	 * @access public
	 * @param  WP_REST_Request $request Full details about the request.
	 * @return WP_REST_Response
	 */
	public function save_token_callback( $request ) {
		$token        = ! empty( $request['source_id'] ) ? wc_clean( $request['source_id'] ) : 0;
		$user_id      = ! empty( $request['user_id'] ) ? wc_clean( $request['user_id'] ) : 0;
		$cc_type      = ! empty( $request['cc_type'] ) ? wc_clean( $request['cc_type'] ) : '';
		$payment_type = ! empty( $request['payment_type'] ) ? wc_clean( $request['payment_type'] ) : '';
		$gateway_id   = ! empty( $request['gateway'] ) ? wc_clean( $request['gateway'] ) : '';
		$last4        = ! empty( $request['last_4'] ) ? wc_clean( $request['last_4'] ) : '';
		$exp_month    = ! empty( $request['exp_month'] ) ? wc_clean( $request['exp_month'] ) : '';
		$exp_year     = ! empty( $request['exp_year'] ) ? wc_clean( $request['exp_year'] ) : '';

		$cc_types = array(
			'mastercard',
			'visa',
			'discover',
			'american express',
			'diners',
			'jcb',
		);

		/**
		 * Check if card type is valid.
		*/
		if ( ! empty( $cc_type ) && ! in_array( $cc_type, $cc_types ) ) {
			return new WP_Error( 'perfectcheckout_card_not_valid', __( 'Card type is not valid. Acceptable values are: "mastercard, visa, discover, american express, diners, jcb"', 'perfect-checkout' ), array( 'status' => 400 ) );
		}

		/**
		 * Check source ID was provided.
		 */
		if ( empty( $token ) ) {
			return new WP_Error( 'perfectcheckout_no_source_provided', __( 'No source ID provided to save payment.', 'perfect-checkout' ), array( 'status' => 404 ) );
		}

		/**
		 * Check user ID was provided.
		 */
		if ( empty( $user_id ) ) {
			return new WP_Error( 'perfectcheckout_no_user_found', __( 'No user provided to save payment found.', 'perfect-checkout' ), array( 'status' => 404 ) );
		}

		// Get all previous tokens for user.
		$current_tokens = Perfect_Checkout_Payment_Tokens::get_customer_tokens( 
			array(
				'user_id' => $user_id
			)
		);

		// Get user details.
		$user = get_user_by( 'id', $user_id );

		// Add token to WooCommerce.
		$fields_provided = false;
		$saved           = false;
		$wc_token        = false;

		/**
		 * Save payment token.
		 */
		if ( $user && class_exists( 'WC_Payment_Token_CC' ) ) {
			if ( ! empty( $payment_type ) ) {
				switch ( $payment_type ) {
					case 'alipay':
						break;
					case 'sepa_debit':
						if ( class_exists( 'WC_Payment_Token_SEPA' ) ) {
							$wc_token = new WC_Payment_Token_SEPA();
							$wc_token->set_token( $token );
							$wc_token->set_gateway_id( 'sepa_debit' );
							$wc_token->set_last4( $last4 );

							$fields_provided = true;
						}
						break;
					default:
						$wc_token = new WC_Payment_Token_CC();
						$wc_token->set_token( $token );
						$wc_token->set_gateway_id( $gateway_id );
						$wc_token->set_card_type( strtolower( $cc_type ) );
						$wc_token->set_last4( $last4 );
						$wc_token->set_expiry_month( $exp_month );
						$wc_token->set_expiry_year( $exp_year );

						$fields_provided = true;

						break;
				}
			} else {
				// Legacy.
				$wc_token = new WC_Payment_Token_CC();
				$wc_token->set_token( $token );
				$wc_token->set_card_type( strtolower( $cc_type ) );
				$wc_token->set_last4( $last4 );
				$wc_token->set_expiry_month( $exp_month );
				$wc_token->set_expiry_year( $exp_year );
			}

			// Only attempt to save token if fields were set.
			if ( $fields_provided ) {
				$wc_token->set_user_id( $user_id );
				$wc_token->save();

				// If no other tokens exist then set this token as default.
				if ( empty( $current_tokens ) ) {
					// Set token as default for user.
					WC_Payment_Tokens::set_users_default( $user_id, $wc_token->get_id() );
				}

				$saved = true;
			}
		}

		/**
		 * Allows for other methods to store saved tokens if not in core.
		 *
		 * @since 1.4.0
		 */
		do_action( 'pc_saved_token', $request );

		/**
		 * Prepare response.
		 */
		$data = array(
			'saved' => $saved,
			'token' => $wc_token->get_token(),
		);

		// Response.
		$response = rest_ensure_response( $data );
		$response->set_status( 200 );

		return $response;
	} // END save_token_callback()

} // END class
