<?php
/**
 * Perfect Checkout - Manages the API.
 *
 * @author  Sébastien Dumont
 * @package Perfect Checkout\API
 * @since   1.2.0
 * @version 1.4.1
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'Perfect_Checkout_API' ) ) {

	class Perfect_Checkout_API {

		/**
		 * Load me.
		 *
		 * @access  public
		 * @since   1.2.0
		 * @version 1.4.0
		 */
		public function __construct() {
			// REST API was included starting WordPress 4.4.
			if ( ! class_exists( 'WP_REST_Server' ) ) {
				return;
			}

			$this->rest_api_includes();

			// Hook into WordPress ready to init the REST API as needed.
			add_action( 'rest_api_init', array( $this, 'register_rest_routes' ), 10 );

			// Prevent Perfect Checkout from being cached with WP REST API Cache plugin (https://wordpress.org/plugins/wp-rest-api-cache/).
			add_filter( 'rest_cache_skip', array( $this, 'prevent_cache' ), 10, 2 );

			// Force WooCommerce to accept Perfect Checkout requests when authenticating.
			add_filter( 'woocommerce_rest_is_request_to_rest_api', array( $this, 'allow_pc_request_wc_api' ) );

			// Add customer details and store currency to cart response for API v1.
			add_filter( 'cocart_return_cart_contents', array( $this, 'return_customer_details' ), 100, 1 );
			add_filter( 'cocart_return_cart_contents', array( $this, 'return_store_currency_details' ), 100, 1 );
			add_filter( 'cocart_return_empty_cart', array( $this, 'return_customer_details' ), 100, 1 );
			add_filter( 'cocart_return_empty_cart', array( $this, 'return_store_currency_details' ), 100, 1 );

			// Add saved payment methods to cart response for API v1.
			add_filter( 'cocart_return_cart_contents', array( $this, 'saved_payment_methods' ), 101, 1 );
			add_filter( 'cocart_return_empty_cart', array( $this, 'saved_payment_methods' ), 101, 1 );

			// Save payment method for Square payment gateway.
			add_action( 'pc_saved_token', array( $this, 'save_square_token' ), 0, 1 );
		}

		/**
		 * Save payment method for Square payment gateway.
		 *
		 * @access  public
		 * @since   1.4.0
		 * @version 1.4.1
		 */
		public function save_square_token( $request ) {
			// Get parameters from REST request.
			$parameters = $request->get_params();

			if ( 'square' == $parameters['gateway'] ) {
				$meta_key = '_wc_square_credit_card_payment_tokens';

				// Get current saved tokens for the customer.
				$tokens = get_user_meta( $parameters['user_id'], $meta_key, true );
				$tokens = maybe_unserialize( $tokens );

				// Identify if the token is the default.
				$is_default = empty( $tokens ) ? true : false;

				// If no meta or value is empty, create new array to store tokens.
				if ( empty( $tokens ) ) { $tokens = array(); }

				// Add new token.
				$tokens[ $parameters['source_id'] ] = array(
					"type"      => "credit_card",
					"card_type" => $parameters['cc_type'],
					"last_four" => $parameters['last_4'],
					"exp_month" => $parameters['exp_month'],
					"exp_year"  => $parameters['exp_year'],
					"default"   => $is_default,
				);

				// Update customers tokens.
				update_user_meta( $parameters['user_id'], $meta_key, $tokens );
			}
		} // END save_square_token()

		/**
		 * Register REST API routes.
		 *
		 * @access public
		 * @since  1.3.0
		 */
		public function register_rest_routes() {
			foreach ( $this->get_rest_namespaces() as $namespace => $controllers ) {
				foreach ( $controllers as $controller_name => $controller_class ) {
					if ( class_exists( $controller_class ) ) {
						$this->controllers[ $namespace ][ $controller_name ] = new $controller_class();
						$this->controllers[ $namespace ][ $controller_name ]->register_routes();
					}
				}
			}
		} // END register_rest_routes()

		/**
		 * Get API namespaces - new namespaces should be registered here.
		 *
		 * @access protected
		 * @since  1.3.0
		 * @return array List of Namespaces and Main controller classes.
		 */
		protected function get_rest_namespaces() {
			return array(
				'pc/connect' => $this->get_controllers(),
			);
		} // END get_rest_namespaces()

		/**
		 * List of controllers in the pc/connect namespace.
		 *
		 * @access protected
		 * @since  1.3.0
		 * @return array
		 */
		protected function get_controllers() {
			return array(
				'perfectcheckout-connect'     => 'Perfect_Checkout_API_Connect_Store_Controller',
				'perfectcheckout-info'        => 'Perfect_Checkout_API_Info_Controller',
				'perfectcheckout-save-tokens' => 'Perfect_Checkout_API_Save_Tokens_Controller',
				'perfectcheckout-settings'    => 'Perfect_Checkout_API_Settings_Controller',
			);
		} // END get_controllers()

		/**
		 * Include Perfect Checkout REST API controllers.
		 *
		 * @access public
		 * @since  1.3.0
		 */
		public function rest_api_includes() {
			include_once dirname( __FILE__ ) . '/api/class-perfect-checkout-connect-store-controller.php';
			include_once dirname( __FILE__ ) . '/api/class-perfect-checkout-info-controller.php';
			include_once dirname( __FILE__ ) . '/api/class-perfect-checkout-save-tokens-controller.php';
			include_once dirname( __FILE__ ) . '/api/class-perfect-checkout-settings-controller.php';
		} // END rest_api_includes()

		/**
		 * Prevents Perfect Checkout from being cached.
		 *
		 * @access public
		 * @since  1.3.0
		 * @param  bool   $skip ( default: WP_DEBUG ).
		 * @param  string $request_uri Requested REST API.
		 * @return bool   $skip Results to WP_DEBUG or true if Perfect Checkout requested.
		 */
		public function prevent_cache( $skip, $request_uri ) {
			$rest_prefix = trailingslashit( rest_get_url_prefix() );

			if ( strpos( $request_uri, $rest_prefix . 'perfectcheckout/' ) !== false ) {
				return true;
			}

			return $skip;
		} // END prevent_cache()

		/**
		 * Force WooCommerce to accept Perfect Checkout API requests when authenticating.
		 *
		 * @access public
		 * @static
		 * @since  1.3.0
		 * @param  bool $request Current status of allowing WooCommerce request.
		 * @return bool true|$request Status after checking if Perfect Checkout is allowed.
		 */
		public static function allow_pc_request_wc_api( $request ) {
			if ( empty( $_SERVER['REQUEST_URI'] ) ) {
				return false;
			}

			$rest_prefix = trailingslashit( rest_get_url_prefix() );
			$request_uri = esc_url_raw( wp_unslash( $_SERVER['REQUEST_URI'] ) );

			// Check if the request is to the Perfect Checkout API endpoints.
			$pc = ( false !== strpos( $request_uri, $rest_prefix . 'pc/' ) );

			if ( $pc ) {
				return true;
			}

			return $request;
		} // END allow_pc_request_wc_api()

		/**
		 * Returns customer payment methods.
		 *
		 * @access  protected
		 * @since   1.2.0
		 * @version 1.4.0
		 * @param   string $cart_key - The customers ID.
		 * @return  array
		 */
		protected function get_payment_methods( $cart_key ) {
			$payment_tokens = Perfect_Checkout_Payment_Tokens::get_customer_tokens( 
				array(
					'user_id'    => $cart_key,
					'gateway_id' => array_merge( Perfect_Checkout_Payment_Tokens::get_gateway_ids_supported(), Perfect_Checkout_Payment_Tokens::get_gateway_ids_unsupported() )
				)
			);

			$saved_payment_methods = array();

			foreach ( $payment_tokens as $payment_token ) {
				$gateway  = strtolower( $payment_token->gateway_id );
				$type     = strtolower( $payment_token->type );
				$token_id = $payment_token->token_id;

				$saved_payment_methods[ $gateway ][ $token_id ] = array(
					'token'      => $payment_token->token,
					'type'       => $type,
					'label'      => Perfect_Checkout_Payment_Tokens::get_display_name( $token_id, $type ),
					'is_default' => Perfect_Checkout_Payment_Tokens::is_default( $payment_token->is_default ),
				);

				// Credit Card.
				if ( 'cc' === $type ) {
					$card_type                                                         = Perfect_Checkout_Payment_Tokens::get_card_type( $token_id );
					$saved_payment_methods[ $gateway ][ $token_id ]['method']['last4'] = Perfect_Checkout_Payment_Tokens::get_last4( $token_id );
					$saved_payment_methods[ $gateway ][ $token_id ]['method']['brand'] = ( ! empty( $card_type ) ? wc_get_credit_card_type_label( $card_type ) : esc_html__( 'Credit card', 'perfect-checkout' ) );
					$saved_payment_methods[ $gateway ][ $token_id ]['expires']         = Perfect_Checkout_Payment_Tokens::get_expiry_month( $token_id ) . '/' . substr( Perfect_Checkout_Payment_Tokens::get_expiry_year( $token_id ), -2 );
				}

				// eCheck.
				if ( 'echeck' === $type ) {
					$saved_payment_methods[ $gateway ][ $token_id ]['method']['last4'] = Perfect_Checkout_Payment_Tokens::get_last4( $token_id );
					$saved_payment_methods[ $gateway ][ $token_id ]['method']['brand'] = esc_html__( 'eCheck', 'perfect-checkout' );
				}
			}

			$payment_methods = array();

			// Now we remove the token index.
			foreach ( $saved_payment_methods as $gateway => $methods ) {
				foreach( $methods as $method ) {
					$payment_methods[$gateway][] = $method;
				}
			}

			return $payment_methods;
		} // END get_payment_methods()

		/**
		 * Appends the customers saved payment methods to the cart response if they are logged in.
		 *
		 * @access public
		 * @since  1.2.0
		 * @param  array $cart - Cart response before modification.
		 * @return array $cart - Cart response after modification.
		 */
		public function saved_payment_methods( $cart ) {
			$cart_key = $cart['cart_key'];

			$cart['customer']['saved_payment_methods'] = self::get_payment_methods( $cart_key );

			return $cart;
		} // END saved_payment_methods()

		/**
		 * Return customer details and add to the cart response.
		 *
		 * @access  public
		 * @since   1.3.0
		 * @version 1.4.0
		 * @global  $current_user - Current user logged in.
		 */
		public function return_customer_details( $cart ) {
			$cart_key = $cart['cart_key'];

			$cart['customer'] = array(
				'billing_address'  => self::get_customer( $cart_key, 'billing' ),
				'shipping_address' => self::get_customer( $cart_key, 'shipping' ),
				'meta'             => array(),
			);

			if ( ! empty( $cart_key ) ) {
				$user_meta = get_user_meta( $cart_key );

				if ( ! empty( $user_meta ) && is_array( $user_meta ) ) {
					$user_meta = array_map( function( $a ){ return $a[0]; }, $user_meta );
				}

				$meta = array();

				// Allowed meta to return.
				$allowed_meta_keys = array(
					'nickname',
					'first_name',
					'last_name',
					'locale',
					'paying_customer',
					'_stripe_customer_id',
					'wc_square_customer_id'
				);

				foreach ( $allowed_meta_keys as $meta_key ) {
					if ( is_array( $user_meta ) && array_key_exists ( $meta_key, $user_meta ) ) {
						$meta[ $meta_key ] = $user_meta[ $meta_key ];
					} else {
						$meta[ $meta_key ] = "";
					}
				}

				$cart['customer']['meta'] = $meta;
			}

			return $cart;
		} // END return_customer_details()

		/**
		 * Return store currency details and add to the cart response.
		 *
		 * @access public
		 * @since  1.4.0
		 */
		public function return_store_currency_details( $cart ) {
			$cart['currency'] = self::get_store_currency();

			return $cart;
		} // END return_store_currency_details()

		/**
		 * Returns the customers details.
		 *
		 * @access protected
		 * @since  1.3.0
		 * @param  string $cart_key - The customers ID.
		 * @param  string $fields   - The customer fields to return.
		 * @return array  Returns the customer details based on the field requested.
		 */
		protected function get_customer( $cart_key, $fields = 'billing' ) {
			$user_meta = get_user_meta( $cart_key );

			if ( ! empty( $user_meta ) && is_array( $user_meta ) ) {
				$user_meta = array_map( function( $a ){ return $a[0]; }, $user_meta );
			}

			$results = array();

			$meta_keys = array(
				'first_name',
				'last_name',
				'company',
				'address_1',
				'address_2',
				'city',
				'postcode',
				'country',
				'email',
				'phone',
			);

			foreach ( $meta_keys as $meta_key ) {
				if ( is_array( $user_meta ) && array_key_exists ( $fields . '_' . $meta_key, $user_meta ) ) {
					$results[ $fields . '_' . $meta_key ] = $user_meta[ $fields . '_' . $meta_key ];
				} else {
					$results[ $fields . '_' . $meta_key ] = "";
				}
			}

			return $results;
		} // END get_customer()

		/**
		 * Prepares a list of store currency data to return in responses.
		 *
		 * @access public
		 * @since  1.4.0
		 * @return array
		 */
		public function get_store_currency() {
			$position = get_option( 'woocommerce_currency_pos' );
			$symbol   = html_entity_decode( get_woocommerce_currency_symbol() );
			$prefix   = '';
			$suffix   = '';

			switch ( $position ) {
				case 'left_space':
					$prefix = $symbol . ' ';
					break;
				case 'left':
					$prefix = $symbol;
					break;
				case 'right_space':
					$suffix = ' ' . $symbol;
					break;
				case 'right':
					$suffix = $symbol;
					break;
			}

			return array(
				'currency_code'               => get_woocommerce_currency(),
				'currency_symbol'             => $symbol,
				'currency_minor_unit'         => wc_get_price_decimals(),
				'currency_decimal_separator'  => wc_get_price_decimal_separator(),
				'currency_thousand_separator' => wc_get_price_thousand_separator(),
				'currency_prefix'             => $prefix,
				'currency_suffix'             => $suffix,
			);
		} // END get_store_currency()

	} // END class.

} // END if class exists.

return new Perfect_Checkout_API();
