<?php
/**
 * Perfect Checkout - Installation related functions and actions.
 *
 * @author  Sébastien Dumont
 * @package Perfect Checkout\Install
 * @since   1.0.2
 * @version 1.5.0
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'Perfect_Checkout_Install' ) ) {

	class Perfect_Checkout_Install {

		/**
		 * Updates and callbacks that need to be run per version.
		 *
		 * @access private
		 * @static
		 * @since  1.5.0
		 * @var    array
		 */
		private static $updates = array(
			'1.5.0' => array(
				'perfectcheckout_update_150_options',
			),
		);

		/**
		 * Constructor.
		 *
		 * @access public
		 */
		public function __construct() {
			// Checks version of Perfect Checkout and install/update if needed.
			add_action( 'init', array( $this, 'check_version' ), 5 );

			add_action( 'perfectcheckout_run_update_callback', array( $this, 'run_update_callback' ) );

			// Redirect to License page once activated.
			add_action( 'activated_plugin', array( $this, 'redirect_license' ), 10, 2 );
		} // END __construct()

		/**
		 * Check plugin version and run the updater if necessary.
		 *
		 * This check is done on all requests and runs if the versions do not match.
		 *
		 * @access public
		 * @static
		 */
		public static function check_version() {
			if ( ! defined( 'IFRAME_REQUEST' ) && version_compare( get_option( 'perfectcheckout_version' ), PERFECTCHECKOUT_VERSION, '<' ) && current_user_can( 'install_plugins' ) ) {
				self::install();
				do_action( 'perfectcheckout_updated' );
			}
		} // END check_version()

		/**
		 * Install Perfect Checkout.
		 *
		 * @access  public
		 * @static
		 * @since   1.0.2
		 * @version 1.5.0
		 */
		public static function install() {
			if ( ! is_blog_installed() ) {
				return;
			}

			// Check if we are not already running this routine.
			if ( 'yes' === get_transient( 'perfectcheckout_installing' ) ) {
				return;
			}

			// If we made it till here nothing is running yet, lets set the transient now for five minutes.
			set_transient( 'perfectcheckout_installing', 'yes', MINUTE_IN_SECONDS * 5 );
			if ( ! defined( 'PERFECTCHECKOUT_INSTALLING' ) ) {
				define( 'PERFECTCHECKOUT_INSTALLING', true );
			}

			// Set default settings.
			self::set_default_settings();

			// Set activation date.
			self::set_install_date();

			// Maybe update options.
			self::maybe_update_options();

			// Update plugin version.
			self::update_version();

			delete_transient( 'perfectcheckout_installing' );

			do_action( 'perfectcheckout_installed' );
		} // END install()

		/**
		 * Sets default settings for Perfect Checkout.
		 *
		 * @access public
		 * @static
		 * @return void
		 */
		public static function set_default_settings() {
			// "Hide shipping costs until an address is entered" is disabled.
			update_option( 'woocommerce_shipping_cost_requires_address', 'no' );
		} // set_default_settings()

		/**
		 * Create page on installation.
		 *
		 * @access public
		 * @static
		 */
		public static function maybe_create_page() {
			$page_id = get_option( 'perfectcheckout_page_id' );

			// If page does not already exist then create "Perfect Checkout" page.
			if ( empty( $page_id ) ) {
				self::create_page();
			}
		} // END maybe_create_page()

		/**
		 * Get checkout page details.
		 *
		 * @access  public
		 * @static
		 * @since   1.0.2
		 * @version 1.5.0
		 * @return  array $checkout
		 */
		public static function get_page_details() {
			$checkout = array(
				'name'    => _x( 'perfect-checkout', 'Page slug', 'perfect-checkout' ),
				'title'   => _x( 'Perfect Checkout', 'Page title', 'perfect-checkout' ),
				'content' => '<!-- wp:shortcode -->[woocommerce_checkout]<!-- /wp:shortcode -->',
			);

			return $checkout;
		} // END get_page_details()

		/**
		 * Create cart page that this plugin relies on.
		 *
		 * @access  public
		 * @static
		 * @since   1.0.2
		 * @version 1.5.0
		 * @param   string $slug
		 */
		public static function create_page( $slug = '' ) {
			$checkout = self::get_page_details();

			// If the slug is not empty then set it as the page slug.
			if ( ! empty( $slug ) ) {
				$checkout['name'] = $slug;
			}

			$pc_page_id = self::pc_create_page(
				esc_sql( $checkout['name'] ),
				'perfectcheckout_page_id',
				$checkout['title'],
				$checkout['content']
			);

			if ( $pc_page_id ) {
				Perfect_Checkout_Logger::log( 'New Perfect Checkout page created.', 'info' );
			} else {
				Perfect_Checkout_Logger::log( 'Perfect Checkout did not create!', 'error' );
			}
		} // END create_page()

		/**
		 * Create a page and store the ID in an option.
		 *
		 * @access public
		 * @static
		 * @since  1.5.0
		 * @param  mixed  $slug Slug for the new page.
		 * @param  string $option Option name to store the page's ID.
		 * @param  string $page_title (default: '') Title for the new page.
		 * @param  string $page_content (default: '') Content for the new page.
		 * @param  int    $post_parent (default: 0) Parent for the new page.
		 * @param  string $post_status (default: publish) The post status of the new page.
		 * @return int page ID.
		 */
		public static function pc_create_page( $slug, $option = '', $page_title = '', $page_content = '', $post_parent = 0, $post_status = 'publish' ) {
			global $wpdb;

			$option_value = get_option( $option );

			if ( $option_value > 0 ) {
				$page_object = get_post( $option_value );

				if ( $page_object && 'page' === $page_object->post_type && ! in_array( $page_object->post_status, array( 'pending', 'trash', 'future', 'auto-draft' ), true ) ) {
					// Valid page is already in place.
					return $page_object->ID;
				}
			}

			// Search for an existing page with the specified page slug.
			$valid_page_found = $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM $wpdb->posts WHERE post_type='page' AND post_status NOT IN ( 'pending', 'trash', 'future', 'auto-draft' )  AND post_name = %s LIMIT 1;", $slug ) );

			if ( $valid_page_found ) {
				if ( $option ) {
					update_option( $option, $valid_page_found );
				}
				return $valid_page_found;
			}

			// Search for a matching valid trashed page.
			$trashed_page_found = $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM $wpdb->posts WHERE post_type='page' AND post_status = 'trash' AND post_name = %s LIMIT 1;", $slug ) );

			if ( $trashed_page_found ) {
				$page_id   = $trashed_page_found;
				$page_data = array(
					'ID'          => $page_id,
					'post_status' => $post_status,
				);
				wp_update_post( $page_data );
			} else {
				$page_data = array(
					'post_status'    => $post_status,
					'post_type'      => 'page',
					'post_author'    => 1,
					'post_name'      => $slug,
					'post_title'     => $page_title,
					'post_content'   => $page_content,
					'post_parent'    => $post_parent,
					'comment_status' => 'closed',
				);
				$page_id   = wp_insert_post( $page_data );
			}

			if ( $option ) {
				update_option( $option, $page_id );
			}

			return $page_id;
		} // END pc_create_page()

		/**
		 * Update plugin version to current.
		 *
		 * @access private
		 * @static
		 */
		private static function update_version() {
			update_option( 'perfectcheckout_version', PERFECTCHECKOUT_VERSION );
		} // END update_version()

		/**
		 * Set the time the plugin was installed.
		 *
		 * @access public
		 * @static
		 */
		public static function set_install_date() {
			add_option( 'perfectcheckout_install_date', time() );
		} // END set_install_date()

		/**
		 * Redirects to the License page upon plugin activation.
		 *
		 * @access public
		 * @static
		 * @param  string $plugin             Activate plugin file.
		 * @param  bool   $network_activation Whether to enable the plugin for all sites in the network
		 *                                    or just the current site. Multisite only.
		 */
		public static function redirect_license( $plugin, $network_activation ) {
			// Prevent redirect if plugin name does not match or multiple plugins are being activated.
			if ( plugin_basename( PERFECTCHECKOUT_FILE ) !== $plugin || isset( $_GET['activate-multi'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
				return;
			}

			// If Perfect Checkout has already been installed before then don't redirect.
			if ( ! empty( get_option( 'perfectcheckout_version' ) ) && ! empty( get_option( 'perfectcheckout_install_date', time() ) ) ) {
				return;
			}

			$page = admin_url( 'admin.php' );

			$license = add_query_arg(
				array(
					'page' => 'perfect-checkout',
				),
				$page
			);

			/**
			 * Should Perfect Checkout be installed via WP-CLI,
			 * display a link to the License page.
			 */
			if ( defined( 'WP_CLI' ) && WP_CLI ) {
				WP_CLI::log(
					WP_CLI::colorize(
						'%y' . sprintf( '🎉 %1$s %2$s', __( 'Connect your %3$s account here:', 'perfect-checkout' ), $license, 'Perfect Checkout' ) . '%n'
					)
				);
				return;
			}

			wp_safe_redirect( $license );
			exit;
		} // END redirect_license()

		/**
		 * Get list of update callbacks.
		 *
		 * @access public
		 * @static
		 * @since  1.5.0
		 * @return array
		 */
		public static function get_update_callbacks() {
			return self::$updates;
		} // END get_update_callbacks()

		/**
		 * See if we need to run updates during install.
		 *
		 * @access private
		 * @static
		 * @since  1.5.0
		 */
		private static function maybe_update_options() {
			if ( self::needs_options_updated() ) {
				self::update_options();
			}
		} // END maybe_update_options()

		/**
		 * Is an update needed?
		 *
		 * @access public
		 * @static
		 * @since  1.5.0
		 * @return boolean
		 */
		public static function needs_options_updated() {
			$current_version = get_option( 'perfectcheckout_version', null );
			$updates         = self::get_update_callbacks();
			$update_versions = array_keys( $updates );
			usort( $update_versions, 'version_compare' );

			return ! is_null( $current_version ) && version_compare( $current_version, end( $update_versions ), '<' );
		} // END needs_options_updated()

		/**
		 * Push all needed updates to the queue for processing.
		 *
		 * @access private
		 * @static
		 * @since  1.5.0
		 */
		private static function update_options() {
			$current_version = get_option( 'perfectcheckout_version' );
			$loop            = 0;

			foreach ( self::get_update_callbacks() as $version => $update_callbacks ) {
				if ( version_compare( $current_version, $version, '<' ) ) {
					foreach ( $update_callbacks as $update_callback ) {
						WC()->queue()->schedule_single(
							time() + $loop,
							'perfectcheckout_run_update_callback',
							array(
								'update_callback' => $update_callback,
							),
							'perfectcheckout-updates'
						);
						$loop++;
					}
				}
			}
		} // END update_options()

		/**
		 * Run an update callback when triggered by ActionScheduler.
		 *
		 * @access public
		 * @static
		 * @since  1.5.0
		 * @param  string $callback Callback name.
		 */
		public static function run_update_callback( $callback ) {
			include_once dirname( __FILE__ ) . '/perfect-checkout-update-functions.php';

			if ( is_callable( $callback ) ) {
				self::run_update_callback_start( $callback );
				$result = (bool) call_user_func( $callback );
				self::run_update_callback_end( $callback, $result );
			}
		} // END run_update_callback()

		/**
		 * Triggered when a callback will run.
		 *
		 * @access protected
		 * @static
		 * @since  1.5.0
		 * @param  string $callback Callback name.
		 */
		protected static function run_update_callback_start( $callback ) {
			define( 'PERFECTCHECKOUT_UPDATING', true );
		} // END run_update_callback_start()

		/**
		 * Triggered when a callback has ran.
		 *
		 * @access protected
		 * @static
		 * @since  1.5.0
		 * @param  string $callback Callback name.
		 * @param  bool   $result Return value from callback. Non-false need to run again.
		 */
		protected static function run_update_callback_end( $callback, $result ) {
			if ( $result ) {
				WC()->queue()->add(
					'perfectcheckout_run_update_callback',
					array(
						'update_callback' => $callback,
					),
					'perfectcheckout-updates'
				);
			}
		} // END run_update_callback_end()

	} // END class.

} // END if class exists.

return new Perfect_Checkout_Install();
