<?php
/**
 * Perfect Checkout - Payment Tokens.
 *
 * @author  Sébastien Dumont
 * @package Perfect Checkout\Payment Tokens
 * @since   1.4.0
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'Perfect_Checkout_Payment_Tokens' ) ) {

	class Perfect_Checkout_Payment_Tokens {

		/**
		 * Hook in methods.
		 */
		public static function init() {
			add_filter( 'woocommerce_saved_payment_methods_list', array( __CLASS__, 'add_unsupported_token_gateways' ), 11, 2 );
			add_action( 'pc_delete_payment_method_square', array( __CLASS__, 'pc_delete_square_token' ), 10, 2 );
		} // END init()

		/**
		 * Returns payment gateway ids that Perfect Checkout supports.
		 *
		 * @access public
		 * @static
		 * @return array List of payment gateways all lowercase.
		 */
		public static function get_gateway_ids_supported() {
			return array(
				'paypal',
				'stripe',
				'stripe_sepa',
			);
		} // END get_gateway_ids_supported()

		/**
		 * Returns payment gateway ids that dont use the Payment Token API.
		 *
		 * @access public
		 * @static
		 * @return array List of payment gateways all lowercase.
		 */
		public static function get_gateway_ids_unsupported() {
			return array(
				'square'
			);
		} // END get_gateway_ids_unsupported()

		/**
		 * Returns an array of objects (stdObject) matching specific token criteria.
		 * Accepts token_id, user_id, gateway_id and type.
		 * Each object should contain the fields token_id, gateway_id, token, user_id, type, is_default.
		 *
		 * @access public
		 * @static
		 * @param  array $args List of accepted args: token_id, user_id, gateway_id, type.
		 * @return array
		 */
		public static function get_customer_tokens( $args ) {
			global $wpdb;

			$args = wp_parse_args(
				$args,
				array(
					'token_id'   => '',
					'user_id'    => '',
					'gateway_id' => '',
					'type'       => '',
				)
			);

			$sql   = "SELECT * FROM {$wpdb->prefix}woocommerce_payment_tokens";
			$where = array( '1=1' );

			if ( $args['token_id'] ) {
				$token_ids = array_map( 'absint', is_array( $args['token_id'] ) ? $args['token_id'] : array( $args['token_id'] ) );
				$where[]   = "token_id IN ('" . implode( "','", array_map( 'esc_sql', $token_ids ) ) . "')";
			}

			if ( $args['user_id'] ) {
				$where[] = $wpdb->prepare( 'user_id = %d', absint( $args['user_id'] ) );
			}

			if ( $args['gateway_id'] ) {
				$gateway_ids = $args['gateway_id'];
				$where[]     = "gateway_id IN ('" . implode( "','", array_map( 'esc_sql', $gateway_ids ) ) . "')";
			}

			if ( $args['type'] ) {
				$where[] = $wpdb->prepare( 'type = %s', $args['type'] );
			}

			// phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
			$token_results = $wpdb->get_results( $sql . ' WHERE ' . implode( ' AND ', $where ) );

			return $token_results;
		} // END get_customer_tokens()

		/**
		 * Get payment token meta data.
		 *
		 * @access public
		 * @static
		 * @return object Returns the following fields: token_id, gateway_id, token, user_id, type, is_default.
		 */
		public static function get_payment_token_meta( $token_id ) {
			$data_store = WC_Data_Store::load( 'payment-token' );

			$meta = $data_store->get_metadata( $token_id );

			return $meta;
		} // END get_payment_token_meta()

		/**
		 * Get type display name.
		 *
		 * @access public
		 * @static
		 * @param  int    $token_id Token ID of payment token stored in database.
		 * @param  string $type Type of payment.
		 * @return string
		 */
		public static function get_display_name( $token_id, $type ) {
			if ( $token_id < 1 || empty( $token_id ) ) {
				return '';
			}

			if ( 'cc' === $type ) {
				return sprintf(
					/* translators: 1: credit card type 2: last 4 digits 3: expiry month 4: expiry year */
					__( '%1$s ending in %2$s (expires %3$s/%4$s)', 'perfect-checkout' ),
					wc_get_credit_card_type_label( self::get_card_type( $token_id ) ),
					self::get_last4( $token_id ),
					self::get_expiry_month( $token_id ),
					substr( self::get_expiry_year( $token_id ), 2 )
				);
			}

			return '';
		} // END get_display_name()

		/**
		 * Returns true or false if payment method is default.
		 *
		 * @access public
		 * @static
		 * @param  int     $token_id Token ID of payment token stored in database.
		 * @return boolean Return true if default else false.
		 */
		public static function is_default( $default ) {
			$default = (bool) $default;

			if ( ! is_bool( $default ) ) {
				return false;
			}

			return $default;
		} // END is_default()

		/**
		 * Returns the card type (mastercard, visa, ...).
		 *
		 * @access public
		 * @static
		 * @param  int    $token_id Token ID of payment token stored in database.
		 * @return string Card type
		 */
		public static function get_card_type( $token_id ) {
			if ( $token_id < 1 || empty( $token_id ) ) {
				return '';
			}

			$meta = (array) self::get_payment_token_meta( $token_id );

			$key = 'card_type';

			return array_key_exists( $key, $meta ) ? $meta[$key][0] : '';
		} // END get_card_type()

		/**
		 * Returns the card expiration year (YYYY).
		 *
		 * @access public
		 * @static
		 * @param  int    $token_id Token ID of payment token stored in database.
		 * @return string Expiration year
		 */
		public static function get_expiry_year( $token_id ) {
			if ( $token_id < 1 || empty( $token_id ) ) {
				return '';
			}

			$meta = (array) self::get_payment_token_meta( $token_id );

			$key = 'expiry_year';

			return array_key_exists( $key, $meta ) ? $meta[$key][0] : '';
		} // END get_expiry_year()

		/**
		 * Returns the card expiration month (MM).
		 *
		 * @access public
		 * @static
		 * @param  int    $token_id Token ID of payment token stored in database.
		 * @return string Expiration month
		 */
		public static function get_expiry_month( $token_id ) {
			if ( $token_id < 1 || empty( $token_id ) ) {
				return '';
			}

			$meta = (array) self::get_payment_token_meta( $token_id );

			$key = 'expiry_month';

			return array_key_exists( $key, $meta ) ? $meta[$key][0] : '';
		} // END get_expiry_month()

		/**
		 * Returns the last four digits.
		 *
		 * @access public
		 * @static
		 * @param  int    $token_id Token ID of payment token stored in database.
		 * @return string Last 4 digits
		 */
		public static function get_last4( $token_id ) {
			if ( $token_id < 1 || empty( $token_id ) ) {
				return '';
			}

			$meta = (array) self::get_payment_token_meta( $token_id );

			$key = 'last4';

			return array_key_exists( $key, $meta ) ? $meta[$key][0] : '';
		} // END get_last4()

		/**
		 * Get token results by token ID.
		 *
		 * @access private
		 * @static
		 * @param  int    $token_id Token ID.
		 * @param  object $token_result Token result.
		 * @return null|object Returns valid token information or null if no token can be found.
		 */
		public static function get_token_results( $token_id ) {
			$data_store = WC_Data_Store::load( 'payment-token' );

			$token_result = $data_store->get_token_by_id( $token_id );

			// Still empty? Token doesn't exist? Don't continue.
			if ( empty( $token_result ) ) {
				return null;
			}

			return $token_result;
		} // END get_token_results()

		/**
		 * Process the delete payment method action.
		 *
		 * @access public
		 * @static
		 * @global $wp
		 */
		public static function pc_delete_payment_method_action() {
			global $wp;

			if ( isset( $wp->query_vars['delete-payment-method'] ) && isset( $_GET['pcun'] ) ) {
				wc_nocache_headers();

				$token_id = absint( $wp->query_vars['delete-payment-method'] );
				$token    = (object) self::get_token_results( $token_id );

				$user_id = (int) $token->user_id;

				if ( is_null( $token ) || get_current_user_id() !== $user_id || ! isset( $_REQUEST['_wpnonce'] ) || false === wp_verify_nonce( wp_unslash( $_REQUEST['_wpnonce'] ), 'delete-payment-method-' . $token_id ) ) { // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
					wc_add_notice( __( 'Invalid payment method.', 'perfect-checkout' ), 'error' );
				} else {
					self::delete_saved_token( $user_id, intval( $token_id ) );

					// Allows for other methods to delete payment method if not using core method.
					do_action( 'pc_delete_payment_method_' . $token->gateway_id, $user_id, $token_id );

					wc_add_notice( __( 'Payment method deleted.', 'perfect-checkout' ) );
				}

				wp_safe_redirect( wc_get_account_endpoint_url( 'payment-methods' ) );
				exit();
			}
		} // END pc_delete_payment_method_action()

		/**
		 * Remove a payment token from the database by ID.
		 *
		 * @access public
		 * @static
		 * @param int $user_id User ID.
		 * @param int $token_id Token ID.
		 */
		public static function delete_saved_token( $user_id, $token_id ) {
			global $wpdb;

			$results = $wpdb->get_results(
				$wpdb->prepare( "DELETE FROM {$wpdb->prefix}woocommerce_payment_tokens WHERE token_id = %d AND user_id = %d", esc_sql( $token_id ), absint( $user_id ) )
			);

			// If results were true then remove token meta data.
			if ( $results ) {
				$wpdb->get_results(
					$wpdb->prepare( "DELETE FROM {$wpdb->prefix}woocommerce_payment_tokenmeta WHERE payment_token_id = %d", esc_sql( $token_id ) )
				);
			}
		} // END delete_saved_token()

		/**
		 * Delete Square token from customer.
		 *
		 * @access public
		 * @static
		 */
		public static function pc_delete_square_token( $user_id, $token_id ) {
			$meta_key = '_wc_square_credit_card_payment_tokens';

			// Get current saved tokens for the customer.
			$tokens = get_user_meta( $user_id, $meta_key, true );

			// Remove token from saved tokens.
			unset( $tokens[$token_id] );

			// Update customers tokens.
			update_user_meta( $user_id, $meta_key, $tokens );
		} // END pc_delete_square_token()

		/**
		 * Add unsupported saved methods to the payment methods list.
		 *
		 * @access public
		 * @static
		 * @param int $customer_id Customer ID.
		 * @return array
		 */
		public static function add_unsupported_token_gateways( $list, $customer_id ) {
			$payment_tokens = self::get_customer_tokens(
				array(
					'user_id'    => $customer_id,
					'gateway_id' => self::get_gateway_ids_unsupported()
				)
			);

			foreach ( $payment_tokens as $payment_token ) {
				$token_id        = $payment_token->token_id;
				$delete_url      = add_query_arg( array( 'pcun' => true ), wc_get_endpoint_url( 'delete-payment-method', $token_id ) );
				$delete_url      = wp_nonce_url( $delete_url, 'delete-payment-method-' . $token_id );
				$set_default_url = wc_get_endpoint_url( 'set-default-payment-method', $token_id );
				$set_default_url = wp_nonce_url( $set_default_url, 'set-default-payment-method-' . $token_id );

				$type = strtolower( $payment_token->type );

				$list[ $payment_token->gateway_id ][ $token_id ] = array(
					'method'     => array(
						'gateway' => $payment_token->gateway_id,
					),
					'expires'    => esc_html__( 'N/A', 'perfect-checkout' ),
					'is_default' => self::is_default( $payment_token->is_default ),
					'actions'    => array(
						'delete' => array(
							'url'  => $delete_url,
							'name' => esc_html__( 'Delete', 'perfect-checkout' ),
						),
					),
				);

				// Credit Card.
				if ( 'cc' === $type ) {
					$card_type                                                          = self::get_card_type( $token_id );
					$list[ $payment_token->gateway_id ][ $token_id ]['method']['last4'] = self::get_last4( $token_id );
					$list[ $payment_token->gateway_id ][ $token_id ]['method']['brand'] = ( ! empty( $card_type ) ? wc_get_credit_card_type_label( $card_type ) : esc_html__( 'Credit card', 'perfect-checkout' ) );
					$list[ $payment_token->gateway_id ][ $token_id ]['expires']         = self::get_expiry_month( $token_id ) . '/' . substr( self::get_expiry_year( $token_id ), -2 );
				}

				// eCheck.
				if ( 'echeck' === $type ) {
					$list[ $payment_token->gateway_id ][ $token_id ]['method']['last4'] = self::get_last4( $token_id );
					$list[ $payment_token->gateway_id ][ $token_id ]['method']['brand'] = esc_html__( 'eCheck', 'perfect-checkout' );
				}

				/*if ( "0" === $payment_token->is_default ) {
					$list[ $payment_token->gateway_id ][ $token_id ]['actions']['default'] = array(
						'url'  => $set_default_url,
						'name' => esc_html__( 'Make default', 'perfect-checkout' ),
					);
				}*/
			}

			return $list;
		} // END add_unsupported_token_gateways()

	} // END class

	Perfect_Checkout_Payment_Tokens::init();

} // END class exists
