<?php
/**
 * Loads CoCart packages from the /packages directory.
 *
 * @author  Sébastien Dumont
 * @package CoCart Commercial/Src/Packages
 * @license MIT
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! defined( 'COCART_WHITE_LABEL' ) ) {
	define( 'COCART_WHITE_LABEL', true );
}

/**
 * Packages class.
 */
class CoCart_Packages {

	/**
	 * Constructor.
	 *
	 * @access public
	 */
	private function __construct() {}

	/**
	 * Array of default package names.
	 *
	 * DO NOT EDIT!
	 *
	 * @access protected
	 * @static
	 * @var   array
	 */
	protected static $default_packages = array(
		'rest-api',
		// 'framework',
	);

	/**
	 * Array of package names and their main package classes.
	 *
	 * @access protected
	 * @static
	 * @var   array Key is the package name/directory, value is the main package class which handles init.
	 */
	protected static $packages = array(
		'rest-api'  => 'CoCart',
		'extras'    => 'CoCart_Get_Cart_Enhanced',
		//'framework' => 'CoCart_Pro',
	);

	/**
	 * Init the package loader.
	 *
	 * @access public
	 * @static
	 */
	public static function init() {
		self::check_packages();
		self::load_packages();
	}

	/**
	 * Checks a package exists by looking for it's directory.
	 *
	 * @access public
	 * @static
	 * @param  string $package Package name.
	 * @return boolean
	 */
	public static function package_exists( $package ) {
		return file_exists( dirname( __DIR__ ) . '/packages/' . $package );
	}

	/**
	 * Returns the default packages that should be available.
	 *
	 * @access public
	 * @static
	 * @return array
	 */
	public static function get_default_packages() {
		return self::$default_packages;
	}

	/**
	 * Returns the packages available.
	 *
	 * @access public
	 * @static
	 * @return array
	 */
	public static function get_packages() {
		return self::$packages;
	}

	/**
	 * Checks packages after plugins_loaded hook.
	 *
	 * @access protected
	 * @static
	 */
	protected static function check_packages() {
		$packages = array();

		foreach ( self::get_packages() as $package_name => $package_class ) {
			// Warn user if package is missing!
			if ( ! self::package_exists( $package_name ) ) {
				if ( 'rest-api' === $package_name ) {
					self::missing_core_package();
				} else {
					$packages[] = $package_name;
				}
			}
		}

		if ( count( $packages ) > 0 ) {
			self::missing_packages( $packages );
		}
	} // END check_packages()

	/**
	 * Loads packages after plugins_loaded hook.
	 *
	 * Each package should include an init file which loads the package so it can be used by core.
	 *
	 * @access protected
	 * @static
	 */
	protected static function load_packages() {
		foreach ( self::get_packages() as $package_name => $package_class ) {
			// Check package is not missing!
			if ( self::package_exists( $package_name ) ) {
				// Load Package.
				include_once dirname( __DIR__ ) . '/packages/' . $package_name . '/load-package.php';

				// Call Package.
				if ( class_exists( $package_class ) ) {
					call_user_func( array( $package_class, 'init' ) );

					// Install CoCart if main package found.
					if ( 'rest-api' === $package_name ) {
						call_user_func( array( $package_class, 'install_cocart' ), true );
					}
				}
			}
		}
	} // END load_packages()

	/**
	 * If the core package is missing, add an admin notice.
	 *
	 * @access protected
	 */
	protected static function missing_core_package() {
		if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {
			error_log(
				sprintf(
					esc_html__( 'Your missing the core of %s!', 'cocart-commercial' ),
					'CoCart'
				) . ' - ' . esc_html__( 'Your installation of CoCart Commercial is incomplete. If you installed CoCart Commercial via Composer, please refer to this document to config your packages: #LINK GOES HERE#', 'cocart-commercial' )
			);
		}
		add_action(
			'admin_notices',
			function() {
				?>
				<div class="notice notice-error">
					<p>
						<strong>
							<?php
							printf(
								esc_html__( 'Your missing the core of %s!', 'cocart-commercial' ),
								'CoCart Commercial'
							);
							?>
						</strong>
						<br>
						<?php
						printf(
							/* translators: 1: is a link to a support document. 2: closing link */
							esc_html__( 'Your installation of CoCart Commercial is incomplete. If you installed CoCart Commercial via Composer, %1$splease refer to this document%2$s to config your packages.', 'cocart-commercial' ),
							'<a href="' . esc_url( '#' ) . '" target="_blank" rel="noopener noreferrer">',
							'</a>'
						);
						?>
					</p>
				</div>
				<?php
			}
		);
	} // END missing_core_package()

	/**
	 * If a package is missing, add an admin notice.
	 *
	 * @access protected
	 * @param  string $packages Package names.
	 */
	protected static function missing_packages( $packages ) {
		$packages = implode( '</code>, <code>', $packages );

		if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {
			error_log(
				sprintf(
					/* Translators: %s package name. */
					esc_html__( 'Your missing the following CoCart Commercial packages: %s', 'cocart-commercial' ),
					'<code>' . $packages . '</code>'
				) . ' - ' . esc_html__( 'Your installation of CoCart Commercial is incomplete. If you installed CoCart Commercial via Composer, please refer to this document to config your packages: #LINK GOES HERE#', 'cocart-commercial' )
			);
		}
		add_action(
			'admin_notices',
			function() use ( $packages ) {
				?>
				<div class="notice notice-error">
					<p>
						<strong>
							<?php
							printf(
								/* Translators: %s package names. */
								esc_html__( 'Your missing the following CoCart Commercial packages: %s', 'cocart-commercial' ),
								'<code>' . $packages . '</code>'
							);
							?>
						</strong>
						<br>
						<?php
						printf(
							/* translators: 1: is a link to a support document. 2: closing link */
							esc_html__( 'Your installation of CoCart Commercial is incomplete. If you installed CoCart Commercial via Composer, %1$splease refer to this document%2$s to config your packages.', 'cocart-commercial' ),
							'<a href="' . esc_url( '#' ) . '" target="_blank" rel="noopener noreferrer">',
							'</a>'
						);
						?>
					</p>
				</div>
				<?php
			}
		);
	} // END missing_packages()

	/**
	 * Returns the package version.
	 *
	 * @access public
	 * @static
	 * @param  string $package_name - The package name.
	 * @return string $version - The version of the package returned.
	 */
	public static function get_package_version( $package_name ) {
		$packages = self::get_packages();

		$version = null;

		foreach ( $packages as $name => $package ) {
			if ( $name === $package_name ) {
				$version = $package::get_version();
			}
		}

		return $version;
	} // END get_package_version()

} // END class
